{% macro similar_pages(language, similar_pages) %}
  <ul>
    {% for similar_score in similar_pages %}
      {% set similar_page = get_storage().get_page(id=similar_score.page_id) %}
      {% set similar_article = get_storage().get_article(id=similar_page.article_id) %}
      <li data-brigid-similarity-score="{{similar_score.score}}">
        <a href="{{root_url(language).to_post(similar_article.slug).url()}}"
           title="{{ translate_theme(language, 'similarity_score_tooltip') }}: {{similar_score.score}}">
          {{similar_page.title}}
        </a>

        {# <ul> #}
        {#   {% for explanation in similar_score.explanations %} #}
        {#     <li>{{explanation}}</li> #}
        {#   {% endfor %} #}
        {# </ul> #}

      </li>
    {% endfor %}
  </ul>
{% endmacro %}
